#include "91x_lib.h"
#include "91x_gpio.h"
#include "91x_adc.h"
#include "91x_map.h"
#include "91x_scu.h"
#include "91x_tim.h"
#include "91x_rtc.h"

#include "lcd_lib_91x.h"
#include "char_code.h"

#include "FreeRTOS.h"
#include "task.h"
#include "queue.h"
#include "typedefs.h"

#define high_PRIORITY 4

void UART_conf();
void ADC_conf();
void PWM_conf(); 
void VIC_conf();
void RTC_conf();
void LCD_DataLinesConfig();
void LCD_conf();
void TIM1_conf();
void GPIO_conf();
void delayy();
void TIM1_Handler();

static void vMainTask(void *pvParameters);
static void vPidTask(void *pvParameters);

/*void uart_rx_handler(void);*/

void itoa(u32, u8 *);

const portTickType xDelay = 500 / portTICK_RATE_MS;
u8 data_to_send[16];
u32 __volt;
u8 __voltc[10];
u8 cnt;
u8 interflag = 0;
RTC_TIME __rtc_time;
/*
 void uart_rx_handler() {
  /*clear receive interrupt flag*/
  //UART_ClearITPendingBit(UART0, UART_IT_Receive);
  
  /*send back the received character*/
  //while(UART_GetFlagStatus(UART0, UART_FLAG_TxFIFOFull) != RESET);
  //UART_SendData(UART0, 'c');
/*}*/

int main() {
	event_EnableFIQ();
	delayy();
	SCU_MCLKSourceConfig(SCU_MCLK_OSC);

	UART_conf();
	ADC_conf();
	PWM_conf();
	VIC_conf();
	RTC_conf();
	LCD_conf();
	TIM1_conf();
	GPIO_conf();
	GPIO_WriteBit(GPIO2, GPIO_Pin_1, Bit_SET);
	xTaskCreate(vPidTask, "PidTask", configMINIMAL_STACK_SIZE * 4, NULL, high_PRIORITY - 1, NULL);
	vTaskStartScheduler();
	for (;;) ;

	}

static void vPidTask(void *pvParameters) {
	for (;;) {
		ADC_ConversionCmd(ADC_Conversion_Start);	
		while(ADC_GetFlagStatus(ADC_FLAG_ECV) == RESET);
		__volt = (u32)ADC_GetConversionValue(ADC_Channel_4);
		ADC_ClearFlag(ADC_FLAG_ECV);
		TIM_SetPulse(TIM0, TIM_PWM_OC1_Channel, 0xfff  / 1024 * __volt);
		__volt = __volt * 3300;
		__volt >>= 10;
		itoa(__volt, __voltc);
		for (cnt = 0; cnt < strlen(__voltc); cnt++)
			UART_SendData(UART0, __voltc[cnt]);
		while(UART_GetFlagStatus(UART0, UART_FLAG_TxFIFOFull) != RESET);
		for (cnt = 0; cnt < strlen(__voltc); cnt++)
			__voltc[cnt] = '0';
		sprintf(data_to_send, "%d.%d", __volt / 1000, __volt % 1000);
		LCDclr();
		LCDGotoXY(0,0);
		LCDstring(data_to_send,strlen(data_to_send));
		LCDGotoXY(strlen(data_to_send),0);
		LCDsendChar('V');
		if (interflag == 1)
			GPIO_WriteBit(GPIO2, GPIO_Pin_1, Bit_RESET);
		RTC_GetTime(BCD, &__rtc_time);
		sprintf(data_to_send, "%d:%d:%d", BCD2ToBYTE(__rtc_time.hours), BCD2ToBYTE(__rtc_time.minutes), BCD2ToBYTE(__rtc_time.seconds));
		LCDGotoXY(0,1);
		LCDstring(data_to_send,strlen(data_to_send));
		LCDGotoXY(strlen(data_to_send),0);
		LCDcursorOFF();
		vTaskDelay(xDelay);	
	}
}

void GPIO_conf() {
		SCU_APBPeriphReset(__GPIO2, DISABLE);
	SCU_APBPeriphClockConfig(__GPIO2 ,ENABLE);
	
	GPIO_InitTypeDef GPIO_P2;
	
	GPIO_DeInit(GPIO2);
	GPIO_P2.GPIO_Pin = (GPIO_Pin_0 | GPIO_Pin_1);
	GPIO_P2.GPIO_Direction = GPIO_PinOutput;
	GPIO_P2.GPIO_Type = GPIO_Type_PushPull;
   	GPIO_P2.GPIO_Alternate = GPIO_OutputAlt1;
	GPIO_Init(GPIO2, &GPIO_P2);
}
void UART_conf() {
	/* Konfiguracja GPIO3 do UART */
	SCU_APBPeriphReset(__GPIO3, DISABLE);
	SCU_APBPeriphClockConfig(__GPIO3 ,ENABLE);
	GPIO_DeInit(GPIO3);

	GPIO_InitTypeDef GPIO_P3;
	GPIO_P3.GPIO_Pin = GPIO_Pin_0;
	GPIO_P3.GPIO_Direction = GPIO_PinInput;
	GPIO_P3.GPIO_Type = GPIO_Type_PushPull;
	GPIO_P3.GPIO_Alternate = GPIO_InputAlt1;
	GPIO_Init(GPIO3, &GPIO_P3);
	
	GPIO_P3.GPIO_Pin = GPIO_Pin_1;
	GPIO_P3.GPIO_Direction = GPIO_PinOutput;
	GPIO_P3.GPIO_Type = GPIO_Type_PushPull;
	GPIO_P3.GPIO_Alternate = GPIO_OutputAlt2;
	GPIO_Init(GPIO3, &GPIO_P3);
	
	/* Konfiguracja RS-232 */
    	SCU_APBPeriphReset(__UART0,DISABLE);	
  	SCU_APBPeriphClockConfig(__UART0, ENABLE);
	UART_InitTypeDef UART_InitTypeDefU0;
	
	UART_InitTypeDefU0.UART_WordLength = UART_WordLength_8D;
	UART_InitTypeDefU0.UART_StopBits = UART_StopBits_1;
	UART_InitTypeDefU0.UART_Parity = UART_Parity_No;
	UART_InitTypeDefU0.UART_BaudRate = 19200;
	UART_InitTypeDefU0.UART_HardwareFlowControl = UART_HardwareFlowControl_None;
	UART_InitTypeDefU0.UART_Mode = UART_Mode_Tx_Rx;
	UART_InitTypeDefU0.UART_FIFO = UART_FIFO_Enable;
    	UART_InitTypeDefU0.UART_TxFIFOLevel = UART_FIFOLevel_1_2;
    	UART_InitTypeDefU0.UART_RxFIFOLevel = UART_FIFOLevel_1_2; 
	UART_DeInit(UART0);
	/*UART_ITConfig(UART0, UART_IT_Receive, ENABLE);
	VIC0->VAiR[0] = (unsigned int)(uart_rx_handler);
	VIC0->INTSR  &= ~(0x01 << (UART0_ITLine - 16)); /* generate an IRQ interrupt     */
  	//IC0->VCiR[0] =    0x20 | (UART0_ITLine - 16);  /* enable vetored interrupt No 0 */
  	//VIC0->INTER  |=  (0x01 << (UART0_ITLine - 16));
	/*VIC_Config(UART0_ITLine, VIC_IRQ, 0);
  	VIC_ITCmd(UART0_ITLine, ENABLE);	*/
	UART_Init(UART0, &UART_InitTypeDefU0);
	UART_Cmd(UART0, ENABLE);
}

void ADC_conf() {
	/* Konfiguracja GPIO4.0 do ADC */
	SCU_APBPeriphReset(__GPIO4, DISABLE);
	SCU_APBPeriphClockConfig(__GPIO4 ,ENABLE);	
	GPIO_InitTypeDef GPIO_ADC;	
	GPIO_ADC.GPIO_Pin = GPIO_Pin_4;
	GPIO_ADC.GPIO_Direction = GPIO_PinInput;
	GPIO_Init(GPIO4, &GPIO_ADC);
	GPIO_ANAPinConfig(GPIO_ANAChannel4, ENABLE);

	ADC_DeInit();
	/* Konfiguracja przetwornika A/C */
	SCU_APBPeriphReset(__ADC, DISABLE);
	SCU_APBPeriphClockConfig(__ADC ,ENABLE);
	ADC_InitTypeDef ADC_Init_Str;
	
	ADC_Init_Str.ADC_Channel_4_Mode = ADC_NoThreshold_Conversion;
	ADC_Init_Str.ADC_Select_Channel = ADC_Channel_4;
	ADC_Init_Str.ADC_Scan_Mode = DISABLE;
	ADC_Init_Str.ADC_Conversion_Mode = ADC_Single_Mode;
	ADC_Cmd(ENABLE);
    	ADC_PrescalerConfig(0x2);
	ADC_Init(&ADC_Init_Str);
}

void PWM_conf() {
		/* Konfiguracja GPIO6.0 do PWM'a */
	SCU_APBPeriphReset(__GPIO6, DISABLE);
	SCU_APBPeriphClockConfig(__GPIO6 ,ENABLE);
	GPIO_InitTypeDef GPIO_P6;
	GPIO_DeInit(GPIO6);
	GPIO_P6.GPIO_Pin = GPIO_Pin_0;
	GPIO_P6.GPIO_Direction = GPIO_PinOutput;
	GPIO_P6.GPIO_Type = GPIO_Type_PushPull;
	GPIO_P6.GPIO_Alternate = GPIO_OutputAlt2;
	GPIO_Init(GPIO6, &GPIO_P6);	

	/* Konfiguracja timera jako PWM */
	SCU_APBPeriphClockConfig(__TIM01, ENABLE);	
	TIM_InitTypeDef Time_Init_0;
	Time_Init_0.TIM_Mode = TIM_PWM;
	Time_Init_0.TIM_Clock_Source = TIM_CLK_APB;
	Time_Init_0.TIM_Prescaler = 0xff;
	Time_Init_0.TIM_Pulse_Level_1 = TIM_HIGH;
	Time_Init_0.TIM_Period_Level = TIM_LOW;
	Time_Init_0.TIM_Pulse_Length_1 = 0x1FF;
	Time_Init_0.TIM_Full_Period = 0xFFF;
	TIM_DeInit(TIM0);
//	TIM_SetPulse(TIM0, TIM_PWM_OC1_Channel,0x0);
  	TIM_CounterCmd(TIM0, TIM_START);	
	TIM_Init(TIM0, &Time_Init_0);
}

void TIM1_conf() {
	SCU_APBPeriphClockConfig(__TIM01, ENABLE);	
	TIM_InitTypeDef Time_Init_1;
	Time_Init_1.TIM_Mode = TIM_OCM_CHANNEL_1;
	Time_Init_1.TIM_OC1_Modes = TIM_TIMING;
 	Time_Init_1.TIM_Clock_Source = TIM_CLK_APB;         
  	Time_Init_1.TIM_Clock_Edge = TIM_CLK_EDGE_FALLING;  
  	Time_Init_1.TIM_Prescaler = 0x20;                  
  	Time_Init_1.TIM_Pulse_Length_1 = 0x100;          
 	TIM_Init(TIM1, &Time_Init_1);

  	/*VIC_Config(TIM1_ITLine, VIC_FIQ, 0);
  	VIC_ITCmd(TIM1_ITLine, ENABLE);
	TIM_ITConfig(TIM1, TIM_IT_TO, ENABLE);
  	TIM_CounterCmd(TIM1, TIM_START);*/
	}

void VIC_conf() {
	SCU_APBPeriphReset(__VIC, DISABLE);
	SCU_APBPeriphClockConfig(__VIC, ENABLE);
	VIC_DeInit();
	VIC_InitDefaultVectors();
}

void RTC_conf() {
	RTC_DATE rtcdate;
	RTC_TIME rtctime;

	SCU_APBPeriphClockConfig(__RTC,ENABLE);
	SCU_APBPeriphReset(__RTC,DISABLE);
	RTC_DeInit();
  	RTC_PeriodicIntConfig(RTC_Per_1024Hz); 
	RTC_ITConfig(RTC_IT_Per, ENABLE);
 	VIC_Config(RTC_ITLine, VIC_FIQ, 0);
  	VIC_ITCmd(RTC_ITLine, ENABLE);
	
	rtcdate.century = 20;
	rtcdate.year 	= 12;
	rtcdate.month 	= 11;
	rtcdate.day 	= 16;
	rtcdate.weekday = 5;
	RTC_SetDate(rtcdate);

	rtctime.hours 	= 12;
	rtctime.minutes = 40;
	rtctime.seconds = 0;
	rtctime.milliseconds = 0;
	RTC_SetTime(rtctime);
	
}
void LCD_DataLinesConfig()
{
  /* Private typedef */
  /* Deklaracja struktury */
  GPIO_InitTypeDef GPIO_InitStructure;
		GPIO_DeInit(GPIO8);
    /* Configure D7~D4 data lines in Output Push-Pull mode */
    /* Konfiguracja lini portu 8 jako wyjsc D7-D4 w trybie push-pull */
    GPIO_InitStructure.GPIO_Pin = GPIO_Pin_All;
    GPIO_InitStructure.GPIO_Alternate = GPIO_OutputAlt1;
    GPIO_InitStructure.GPIO_Direction = GPIO_PinOutput;
    GPIO_InitStructure.GPIO_Type = GPIO_Type_PushPull;
    GPIO_Init(GPIO8, &GPIO_InitStructure);
    
    GPIO_InitStructure.GPIO_Pin = GPIO_Pin_3;
    GPIO_InitStructure.GPIO_Alternate = GPIO_OutputAlt1;
    GPIO_InitStructure.GPIO_Direction = GPIO_PinOutput;
    GPIO_InitStructure.GPIO_Type = GPIO_Type_PushPull;
    GPIO_Init(GPIO8, &GPIO_InitStructure);

    GPIO_InitStructure.GPIO_Pin = GPIO_Pin_2;
    GPIO_InitStructure.GPIO_Alternate = GPIO_OutputAlt1;
    GPIO_InitStructure.GPIO_Direction = GPIO_PinOutput;
    GPIO_InitStructure.GPIO_Type = GPIO_Type_PushPull;
    GPIO_Init(GPIO8, &GPIO_InitStructure);

    GPIO_InitStructure.GPIO_Pin = GPIO_Pin_1;
    GPIO_InitStructure.GPIO_Alternate = GPIO_OutputAlt1;
    GPIO_InitStructure.GPIO_Direction = GPIO_PinOutput;
    GPIO_InitStructure.GPIO_Type = GPIO_Type_PushPull;
    GPIO_Init(GPIO8, &GPIO_InitStructure);
      
    GPIO_InitStructure.GPIO_Pin = GPIO_Pin_0;
    GPIO_InitStructure.GPIO_Alternate = GPIO_OutputAlt1;
    GPIO_InitStructure.GPIO_Direction = GPIO_PinOutput;
    GPIO_InitStructure.GPIO_Type = GPIO_Type_PushPull;
    GPIO_Init(GPIO8, &GPIO_InitStructure);
    
    /* Configure RS, E control lines in Output Push-Pull mode */
    /* Konfiguracja lini portu 8 jako wyjsc RS, E w trybie push-pull */
    GPIO_InitStructure.GPIO_Pin = GPIO_Pin_4;
    GPIO_InitStructure.GPIO_Alternate = GPIO_OutputAlt1;
    GPIO_InitStructure.GPIO_Direction = GPIO_PinOutput;
    GPIO_InitStructure.GPIO_Type = GPIO_Type_PushPull;
    GPIO_Init(GPIO8, &GPIO_InitStructure);
    
    GPIO_InitStructure.GPIO_Pin = GPIO_Pin_5;
    GPIO_InitStructure.GPIO_Alternate = GPIO_OutputAlt1;
    GPIO_InitStructure.GPIO_Direction = GPIO_PinOutput;
    GPIO_InitStructure.GPIO_Type = GPIO_Type_PushPull;
    GPIO_Init(GPIO8, &GPIO_InitStructure);
}
void LCD_conf() {
	SCU_APBPeriphReset(__GPIO8, DISABLE);
	SCU_APBPeriphClockConfig(__GPIO8 ,ENABLE);
	
	/* Konfiguracja GPIO8 do LCD */
	LCD_DataLinesConfig();

	/* Wlaczenie LCD */
	LCDinit();
		LCDGotoXY(0,0);
		LCDstring("Zbyt zajebisty..", 16);
		LCDcursorOFF();
	}

void itoa(u32 val, u8 *str) {
	int it, jt;
	char temp;
	for (it = 0; val > 0; val /= 10, it++) {
		str[it] = (char)(val % 10) + 48;
		}
	str[++it] = '\0';
	for (it = 0, jt = strlen(str) - 1; it <= jt; it++, jt--) {
		temp = str[it];
		str[it] = str[jt];
		str[jt] = temp;
	}
	
}


void delayy() {
	u16 it;
	for (it = 0; it < 10000; it++);
}

/*void TIM1_Handler(void)
{
   /*write your handler here*/
   /* ... */
  // TIM_ClearFlag(TIM1, TIM_FLAG_OC1);
  
  /* Reset TIM1 Counter*/
 // TIM_CounterCmd(TIM1, TIM_CLEAR);
        
  // UART_SendData(UART0, 'c');
   /*write any value to VIC0 VAR*/  
   //VIC0->VAR = 0xFF;
//}*/
